/**
 * @module Test_Installation_Process
 * @description
 * This module will run a complete integration test.
 * This file is ran by running <code>npm run test:process</code>
 * After running <code>npm run init</code>, this file will execute the following functions in this order)
 * <pre><code>
 npmRunTest();
 npmRunAnalyze();
 npmRunBuild();
 * </code></pre>
 *
 */


let {breakOnError, execWrap, log, fs, path} = require('./shared/bootstrap');
require('../util/npmPack').npmPack();
let clearPack = require('../util/npmPack').cleanPack;
let {npmRunTest} = require('./npmRunTest');
let {ensureSrcExists} = require('./ensureSrcExists');
let {npmRunAnalyze} = require('./npmRunAnalyze');

// this global test should first of all, init the project!
process.chdir('.test');
// log('npm install...');
// execWrap('npm run init', (err) => {
//
// });

function commonBuildTests() {
  let indexFile = fs.readFileSync('dist/index.js', 'utf8');
  // lets make sure the react component was bundled correctly

  let jsTest1 = indexFile.indexOf('TestString') >= 0; // comes from the index.js itself

  let jsOK = jsTest1;

  // let's make sure the image path is correct
  let isImagePathCorrect = indexFile.indexOf('src/images/image.png');

  if (!isImagePathCorrect) {
    log('Error - Cannot find src/images/image.png in output, ensure this path was not changed in example project!');
    breakOnError();
  }

  let bundleFile = fs.readFileSync('dist/bundle.css', 'utf8');
  // lets make sure the scss was parsed correctly
  let cssOK = bundleFile.indexOf('margin') >= 0;
  // let's make sure duplications were removed!
  // ensure that the original style had duplications
  let originalFile = fs.readFileSync('src/styles/vars.scss', 'utf8');


  if ((originalFile.match(/margin/g) || []).length < 2) {
    log('Error - cannot test duplicate CSS removal - original file has no duplicate css');
    breakOnError();
  }

  if ((bundleFile.match(/margin/g) || []).length > 1) {
    log('Error - Duplicate CSS found in bundle.css - margin');
    breakOnError();
  }  else {
    log('Successfuflly removed duplicate css - margin');
  }

  if ((bundleFile.match(/padding/g) || []).length > 1) {
    log('Error - Duplicate CSS found in bundle.css - padding');
    breakOnError();
  }  else {
    log('Successfuflly removed duplicate css - padding');
  }

  let isScssImagePathOK = bundleFile.indexOf('src/images/scssImage.png') >= 0;

  if (!isScssImagePathOK) {
    log('Error - SCSS imagePath could not found correctly!');
    breakOnError();
  }

  return {
    jsOK,
    cssOK
  };
}
/**
 * Runs <code>npm run build</code> - Ensures dist/index.js and dist/bundle.css are created with proper content
 * This also checks that the webpack prod config is working properly, as we check SCSS and jsx parsing
 */

function npmRunBuild(cb = () => {}) {
  execWrap('npm run build', (err) => {
    let {jsOK, cssOK} = commonBuildTests();
    if (err || !jsOK || !cssOK) {
      log('error running build', {cssOK, jsOK, err});
      breakOnError();
    }

    log('OK - npm run build');
    cb();
  });
}
//
function npmRunBuildWithProjectConfig(cb = () => {}) {
  // create config file
  let projectConfigJson = {
    env : {
      production : {
        assetsPath: '//subdomain.google.co.uk/',
        isMinified : 0
      }
    }
  };

  let assetsPaths = projectConfigJson.env.production.assetsPath;
  fs.writeFileSync('./projectConfig.json', JSON.stringify(projectConfigJson));

  fs.writeFileSync('./src/dist.html', '<html>TESTING</html>');
  // build with config file
  execWrap('npm run build', (err) => {
    let {jsOK, cssOK} = commonBuildTests();

    // extra tests for projectConfig
    // - Do we have ./dist/index.html?
    // - Do we have correct urls in SCSS
    // - do we have correct urls in index.js?
    let hasIndexHtml = fs.readFileSync(path.resolve('./dist/index.html'), 'utf-8').indexOf('TESTING') >= 0;
    let bundleCSS    = fs.readFileSync(path.resolve('./dist/bundle.css'), 'utf-8');
    let indexJS      = fs.readFileSync(path.resolve('./dist/index.js'), 'utf-8');

    let bundleHasBasePath  = bundleCSS.indexOf(`${assetsPaths}src/images/scssImage.png`) >= 0;
    let indexJSHasBasePAth = indexJS.indexOf(`${assetsPaths}src/images/image.png`) >= 0;

    if (err || !jsOK || !cssOK || !hasIndexHtml || !bundleHasBasePath || !indexJSHasBasePAth) {
      log('error running build with packageConfig.json', {
        cssOK,
        jsOK,
        hasIndexHtml,
        err,
        bundleHasBasePath,
        indexJSHasBasePAth});
      breakOnError();
    }
    log('OK - npm run build with packageConfig.json');
    cb();
  });
}
//
//
//
//
//
// // after checking that the src was created correctly, we remove it and use our placeholder
function replaceSrc() {
  fs.renameSync('src', '_________src'); // move the created src away
  fs.renameSync('_src', 'src'); // use the placeholder src instead
}

let testJest = () => {
  let jestConfigPath = path.resolve('./jestConfig.json');
  let content = fs.readFileSync(jestConfigPath, 'utf-8');
  let jestObj = JSON.parse(content);
  return jestObj.testKey === 'success' ? true : breakOnError('Jest configuration creation error');
};

execWrap('npm run init', (err) => {
  // part of the installation, is to make sure the jest file is created successfully.
  // get the object from the jest.conf.json.
  testJest();

  if (err) {
    breakOnError('installation failed');
  }
  log('OK - setup.js');
  ensureSrcExists();
  replaceSrc();

  // // temporary rename tests, as it contains tests related to the example project
  execWrap('echo "STARTING TESTS" && npm run build', (err) => {
    if (err) {
      log('cannot build default project');
      breakOnError();
    }
    log('OK - npm run build on default project');
    npmRunTest();
    npmRunAnalyze();
    npmRunBuild(() => {
      npmRunBuildWithProjectConfig(() => {
        process.chdir('../');
        clearPack();
      });
    });
  });
});
